### Marta Shocket, Indiana University, Stanford University
### Code for 'Parasite rearing and infection temperatures jointly influence disease 
### transmission and shape seasonality of epidemics. Published in Ecology.(2018)

### Purpose: to calculate and bootstrap beta values for field spore infectivity experiment

### Table of Contents:
#      1. Set Working Directory, Load Libraries, Get Data
#      2. Calculate betas
#      3. Bootstrap betas
#      4. Randomization tests
#      5. Figure 3


######
##### 1. Set Working Directory, Load Libraries, Get Data
######

# Set working directory
setwd("~/Dropbox/Research Hall Lab/Temperature/Temp Spore Infectivity")

## Load libraries
library(bbmle)

# Get data of proportions infected
fsi.data = read.csv("FieldSporeInfectivityAssay.csv")


######
##### 2. Calculate betas
######

# Subset data by treatment
nov9 <- subset(fsi.data, Date == "9-Nov")
scott9 <- subset(nov9, Lake == "Scott")
clear9 <- subset(nov9, Lake == "Clear")
gambill9 <- subset(nov9, Lake == "Gambill")

nov23 <- subset(fsi.data, Date == "23-Nov")
clear23 <- subset(nov23, Lake == "Clear")
scott23 <- subset(nov23, Lake == "Scott")
gambill23 <- subset(nov23, Lake == "Gambill")

# Calculate beta values for each treatment - these are untransformed and need to be multiplied by 0.015 host / L
beta.scott9 = mle2(Uninf ~ dbinom(size=Total, prob=exp(-Beta*15*Spore.Dose*1)), start=list(Beta = 0.00004), 
                 control=list(parscale = c(Beta = 0.00001)), data = scott9, method="Brent", lower = 0, upper = .1)
beta.scott23 = mle2(Uninf ~ dbinom(size=Total, prob=exp(-Beta*15*Spore.Dose*1)), start=list(Beta = 0.00004), 
                   control=list(parscale = c(Beta = 0.00001)), data = scott23, method="Brent", lower = 0, upper = .1)
beta.clear9 = mle2(Uninf ~ dbinom(size=Total, prob=exp(-Beta*15*Spore.Dose*1)), start=list(Beta = 0.00004), 
                   control=list(parscale = c(Beta = 0.00001)), data = clear9, method="Brent", lower = 0, upper = .1)
beta.clear23 = mle2(Uninf ~ dbinom(size=Total, prob=exp(-Beta*15*Spore.Dose*1)), start=list(Beta = 0.00004), 
                   control=list(parscale = c(Beta = 0.00001)), data = clear23, method="Brent", lower = 0, upper = .1)
beta.gambill9 = mle2(Uninf ~ dbinom(size=Total, prob=exp(-Beta*15*Spore.Dose*1)), start=list(Beta = 0.00004), 
                   control=list(parscale = c(Beta = 0.00001)), data = gambill9, method="Brent", lower = 0, upper = .1)
beta.gambill23 = mle2(Uninf ~ dbinom(size=Total, prob=exp(-Beta*15*Spore.Dose*1)), start=list(Beta = 0.00004), 
                     control=list(parscale = c(Beta = 0.00001)), data = gambill23, method="Brent", lower = 0, upper = .1)

# Get beta values
coef(beta.scott9)*0.015
coef(beta.scott23)*0.015
coef(beta.clear9)*0.015
coef(beta.clear23)*0.015
coef(beta.gambill9)*0.015
coef(beta.gambill23)*0.015


######
##### 3. Bootstrap betas
######

# Create expanded versions of data for each lake-date to boostrap from

scott9.50 <- c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
               0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
               0, 0)

scott9.150 <- c(1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
               0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
               0)

clear9.50 <- c(1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
               0, 0, 0, 0, 0, 0, 0, 0, 0, 0)

clear9.150 <- c(1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
                0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                0, 0, 0)

gambill9.50 <- c(1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
                 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                 0, 0, 0)

gambill9.150 <- c(1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
                  1, 1, 1, 0, 0, 0, 0, 0, 0)

clear23.50 <- c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                0, 0, 0)

clear23.150 <- c(1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                 0, 0, 0)

scott23.50 <- c(1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                0, 0)

scott23.150 <- c(1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
                 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                 0, 0, 0, 0)

gambill23.50 <- c(1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
                  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                  0, 0)

gambill23.150 <- c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                   0, 0, 0, 0, 0, 0, 0, 0)


################## Function - BootstrapBeta

# Arguments: 
  # data.set.50               (vector: expanded 50 sp/ml data)
  # data.set.150              (vector: expanded 150 sp/ml data)
  # kIterations               (integer: # of bootstraping iterations = number of coefficient sets produced)
# Returns: 
  # function.coefs            (vector: vector of bootstrapped fit coefficients)

BootstrapBeta = function(data.set.50, data.set.150, kIterations) {
  
  # Create data frame for output
  beta.output = numeric(length(kIterations))
  
  # Loop through iterations
  for (i in 1:kIterations) {
    
    # Create data frame for bootstrapped data
    simulated.dataset <- data.frame(Spore.Dose = c(50,150), Uninf = numeric(2), Total = numeric(2))
    
    # Sample 50 sp/ml data
    sample.50 <- sample(data.set.50, length(data.set.50), replace = TRUE)
    
    # Flatten and add sample to data frame
    simulated.dataset[1, 2] <- length(sample.50) - sum(sample.50)
    simulated.dataset[1, 3] <- length(sample.50)
    
    # Sample 150 sp/ml data
    sample.150 <- sample(data.set.150, length(data.set.150), replace = TRUE)
    
    # Flatten and add sample to data frame
    simulated.dataset[2, 2] <- length(sample.150) - sum(sample.150)
    simulated.dataset[2, 3] <- length(sample.150)
    
    # Calculate beta of simulated data set
    beta <- mle2(Uninf ~ dbinom(size=Total, prob=exp(-Beta*15*Spore.Dose*1)), start=list(Beta = 0.00004), 
                 control=list(parscale = c(Beta = 0.00001)), data = simulated.dataset, method="Brent", lower = 0, upper = .1)
    
    # Store beta parameter; ***Multiply by 0.015 L / tube to get correct units (L instead of tubes)***
    beta.output[i] = coef(beta)*0.015
    
  }
  
  # Return output
  beta.output
}

# Bootstrap betas for each lake-date
clear.9.betas <- BootstrapBeta(clear9.50, clear9.150, 10000)
clear.23.betas <- BootstrapBeta(clear23.50, clear23.150, 10000)

gambill.9.betas <- BootstrapBeta(gambill9.50, gambill9.150, 10000)
gambill.23.betas <- BootstrapBeta(gambill23.50, gambill23.150, 10000)

scott.9.betas <- BootstrapBeta(scott9.50, scott9.150, 10000)
scott.23.betas <- BootstrapBeta(scott23.50, scott23.150, 10000)

# Save boostrapped betas
save(clear.9.betas, file = "clear.9.betas.Rdata")
save(clear.23.betas, file = "clear.23.betas.Rdata")

save(gambill.9.betas, file = "gambill.9.betas.Rdata")
save(gambill.23.betas, file = "gambill.23.betas.Rdata")

save(scott.9.betas, file = "scott.9.betas.Rdata")
save(scott.23.betas, file = "scott.23.betas.Rdata")

# Gets quantiles for each lake-date
quantile(clear.9.betas, c(0.025, 0.975))
quantile(clear.23.betas, c(0.025, 0.975))

quantile(gambill.9.betas, c(0.025, 0.975))
quantile(gambill.23.betas, c(0.025, 0.975))

quantile(scott.9.betas, c(0.025, 0.975))
quantile(scott.23.betas, c(0.025, 0.975))


######
##### 4. Randomization tests
######

# Calculate differences in betas from point estimates
coef(beta.scott9)*0.015 - coef(beta.scott23)*0.015
coef(beta.gambill9)*0.015 - coef(beta.gambill23)*0.015
coef(beta.clear9)*0.015 - coef(beta.clear23)*0.015 

# Make data frames for both dates from each lake:
scott.inf <- c(scott9.50, scott9.150, scott23.50, scott23.150)
scott.dose <- c(rep(50, length(scott9.50)), rep(150, length(scott9.150)), rep(50, length(scott23.50)), rep(150, length(scott23.150)))
scott.date <- c(rep("Nov9", length(scott9.50)), rep("Nov9", length(scott9.150)), rep("Nov23", length(scott23.50)), rep("Nov23", length(scott23.150)))
scott.rand.df <- data.frame(infect = scott.inf, Spore.Dose = scott.dose, Date = scott.date)

clear.inf <- c(clear9.50, clear9.150, clear23.50, clear23.150)
clear.dose <- c(rep(50, length(clear9.50)), rep(150, length(clear9.150)), rep(50, length(clear23.50)), rep(150, length(clear23.150)))
clear.date <- c(rep("Nov9", length(clear9.50)), rep("Nov9", length(clear9.150)), rep("Nov23", length(clear23.50)), rep("Nov23", length(clear23.150)))
clear.rand.df <- data.frame(infect = clear.inf, Spore.Dose = clear.dose, Date = clear.date)

gambill.inf <- c(gambill9.50, gambill9.150, gambill23.50, gambill23.150)
gambill.dose <- c(rep(50, length(gambill9.50)), rep(150, length(gambill9.150)), rep(50, length(gambill23.50)), rep(150, length(gambill23.150)))
gambill.date <- c(rep("Nov9", length(gambill9.50)), rep("Nov9", length(gambill9.150)), rep("Nov23", length(gambill23.50)), rep("Nov23", length(gambill23.150)))
gambill.rand.df <- data.frame(infect = gambill.inf, Spore.Dose = gambill.dose, Date = gambill.date)

################## Function - BetaDiffRand

# Arguments: 
  # data.set.50               (data frame: infection, spore dose, and dates for a given lake)
  # kIterations               (integer: number of bootstraping iterations = number of beta difference values produced)
# Returns: 
  # function.coefs            (data frame: output of of randomized beta coefficients and difference)

BetaDiffRand = function(data.set, kIterations) {
  
  # Create data frame for output
  betadiff.output = data.frame(beta.nov9 = numeric(kIterations), beta.nov23 = numeric(kIterations), beta.diff = numeric(kIterations))
  
  # Loop through iterations
  for (i in 1:kIterations) {
    
    # Add randomized dates to data frame
    simulated.dataset <- data.set
    simulated.dataset$Date.rand <- sample(simulated.dataset$Date, replace = FALSE)    
    
    # Subset data frames
    simulated.dataset.nov9.50 <- subset(simulated.dataset, Date.rand == "Nov9" & Spore.Dose == 50)
    simulated.dataset.nov9.150 <- subset(simulated.dataset, Date.rand == "Nov9" & Spore.Dose == 150)
    simulated.dataset.nov23.50 <- subset(simulated.dataset, Date.rand == "Nov23" & Spore.Dose == 50)
    simulated.dataset.nov23.150 <- subset(simulated.dataset, Date.rand == "Nov23" & Spore.Dose == 150)
    
    # Flatten each date for mle calculation
    sim.dataset.nov9.flat <- data.frame(Spore.Dose = c(50,150), Uninf = numeric(2), Total = numeric(2))
    sim.dataset.nov9.flat[1, 2] <- nrow(simulated.dataset.nov9.50) - sum(simulated.dataset.nov9.50$infect)
    sim.dataset.nov9.flat[1, 3] <- nrow(simulated.dataset.nov9.50)
    sim.dataset.nov9.flat[2, 2] <- nrow(simulated.dataset.nov9.150) - sum(simulated.dataset.nov9.150$infect)
    sim.dataset.nov9.flat[2, 3] <- nrow(simulated.dataset.nov9.150)
    
    sim.dataset.nov23.flat <- data.frame(Spore.Dose = c(50,150), Uninf = numeric(2), Total = numeric(2))
    sim.dataset.nov23.flat[1, 2] <- nrow(simulated.dataset.nov23.50) - sum(simulated.dataset.nov23.50$infect)
    sim.dataset.nov23.flat[1, 3] <- nrow(simulated.dataset.nov23.50)
    sim.dataset.nov23.flat[2, 2] <- nrow(simulated.dataset.nov23.150) - sum(simulated.dataset.nov23.150$infect)
    sim.dataset.nov23.flat[2, 3] <- nrow(simulated.dataset.nov23.150)
    
    # Calculate betas of randomized data sets
    beta.nov9 <- mle2(Uninf ~ dbinom(size=Total, prob=exp(-Beta*15*Spore.Dose*1)), start=list(Beta = 0.00004), 
                      control=list(parscale = c(Beta = 0.00001)), data = sim.dataset.nov9.flat, method="Brent", lower = 0, upper = .1)
    
    beta.nov23 <- mle2(Uninf ~ dbinom(size=Total, prob=exp(-Beta*15*Spore.Dose*1)), start=list(Beta = 0.00004), 
                       control=list(parscale = c(Beta = 0.00001)), data = sim.dataset.nov23.flat, method="Brent", lower = 0, upper = .1)
    
    # Store beta parameter; *** multiply by 0.015 L / tube to get correct units (L instead of tubes) ***
    betadiff.output$beta.nov9[i] = coef(beta.nov9)*0.015
    betadiff.output$beta.nov23[i] = coef(beta.nov23)*0.015
    betadiff.output$beta.diff[i] = betadiff.output$beta.nov9[i] - betadiff.output$beta.nov23[i]
    
  }
  
  # Return output
  betadiff.output
}

# Run randomizations
scott.diff <- BetaDiffRand(scott.rand.df, 10000)
clear.diff <- BetaDiffRand(clear.rand.df, 10000)
gambill.diff <- BetaDiffRand(gambill.rand.df, 10000)

# Calculate inverse quantiles (i.e., the proportion of randomized values that are as extreme or lower as the estimated value)
1 - ecdf(scott.diff$beta.diff)(6.309917e-07)
1 - ecdf(clear.diff$beta.diff)(2.088069e-06)
1 - ecdf(gambill.diff$beta.diff)(4.737832e-06 )


######
##### 5. Figure
######
  
# Get data of proportions infected
fsi.out.data = read.csv("FieldSporeInfectivityAssayBetas_Final.csv")

# Subset data by treatment
fsi.out.clear <- subset(fsi.out.data, Lake == "Clear")
betas.downing <- subset(fsi.out.data, Lake == "Downing")
fsi.out.gambill <- subset(fsi.out.data, Lake == "Gambill")
fsi.out.scott <- subset(fsi.out.data, Lake == "Scott")

options(scipen = 10)
par(mfrow = c(1,1), mar = c(7, 5, 1.5, 1.5), mgp = c(3, 1, 0))

plot(Beta*1000000 ~ Date.Num, ylim = c(0, 8.5), xlim = c(0.5, 2.5), data = fsi.out.clear, xlab = "", las = 1, pch = 21, bg = "white",
     ylab = expression(paste("Trans. rate - ",beta," (L spore"^-1," day"^-1,"10"^-6,")")), cex.lab = 1.25, cex.axis = 1, cex = 1.25, xaxt = "n", type = "n")
axis(side = 1, at = c(1, 2), labels = c("", ""), cex.lab = 1.25)
mtext("  Spore collection date \n Weighted temperature", side = 1, line = 5, cex = 1.25)
mtext(expression(paste("14",degree,"C                                    10",degree,"C")), side = 1, line = 2.1, cex = 1.1)
mtext("Nov. 9                                 Nov. 23", side = 1, line = 0.9, cex = 1.1)

arrows(fsi.out.clear$Date.Num, fsi.out.clear$Upper*1000000, fsi.out.clear$Date.Num, fsi.out.clear$Lower*1000000, length = 0, angle = 90, code = 3, lwd = 1)
arrows(fsi.out.gambill$Date.Num, fsi.out.gambill$Upper*1000000, fsi.out.gambill$Date.Num, fsi.out.gambill$Lower*1000000, length = 0, angle = 90, code = 3, lwd = 1)
arrows(fsi.out.scott$Date.Num, fsi.out.scott$Upper*1000000, fsi.out.scott$Date.Num, fsi.out.scott$Lower*1000000, length = 0, angle = 90, code = 3, lwd = 1)

lines(Beta*1000000 ~ Date.Num, data = fsi.out.clear, pch = 21, type = "o", cex = 1.5, bg = "grey", lwd = 1, lty = 2)
lines(Beta*1000000 ~ Date.Num, data = fsi.out.gambill, pch = 21, type = "o", cex = 1.5, bg = "black", lwd = 1, lty = 1)
lines(Beta*1000000 ~ Date.Num, data = fsi.out.scott, pch = 21, type = "o", cex = 1.5, bg = "white", lwd = 1, lty = 3)

legend(x = 1.3, y = 8, legend = c("Gambill", "Clear", "Scott"), 
       bty = "n", pch = c(21, 21, 21), cex = 1, pt.cex = 1.25, pt.bg = c("black", "grey", "white"))
text(x = 1.48, y = 8.1, labels = "Lakes:", cex = 1.1)

text(x = 2.22, y = 7.46, labels = expression(paste("(",italic(p)," < 0.0001)")), cex = .9)
text(x = 2.1, y = 6.87, labels = expression(paste("(",italic(p)," = 0.0024)")), cex = .9)
text(x = 2.07, y = 6.35, labels = expression(paste("(",italic(p)," = 0.16)")), cex = .9)

text(x = 1.5, y = 4, labels = "*", cex = 2)
text(x = 1.5, y = 2, labels = "*", cex = 2)
text(x = 1.5, y = 0.6, labels = "N.S.", cex = .8)